struct Point {
	float x,y;
};

struct Rect {
	int x,y,w,h;
};

class GameObject {
	public:
		Point position;
		Point velocity;
		// dead - set this and the manager will remove the object
		// Next time around main loop.
		// Other objects can test references to make sure objects are
		// still alive.
		// dead objects will not be drawn or ticked
		// but will be kept around for one more cycle so that they can
		// then be forgotten about subsequently.
		bool dead; // Not just dead, but removed totally
		virtual void draw() = 0;
		virtual void tick(float quantum);
		virtual ~GameObject();
		virtual void save(ostream &out);
};

class CollidesWithScenery : public GameObject {
	public:
		float radius;
		float mass;
		virtual void tick(float quantum);
		virtual void takehit(float strength);
		// Called on a collision with another obj.
		virtual void collidewithother(CollidesWithScenery * other, float strength);
		void collidedetect();
};

enum AmmoType {
	Hydrochloric,
	Hydrofluoric
};

class Ship : public CollidesWithScenery {
	public:
		float angle; // Radians? 0=up?
		float thrust;
		float shieldstress; // how bright to draw shield
		float shieldpower;
		BITMAP * sprite;
		Ship(); // Default constructor.
		virtual void draw();
		virtual void tick(float quantum);
		virtual ~Ship();
		virtual void save(ostream &out);
		virtual void takehit(float strength);
		virtual void collidewithother(CollidesWithScenery * other, float strength);
		float hydrogen; // 0..1
		float oxygen; // 0..1
		float ammo; // number of shots (Acid)
		AmmoType ammotype;
	private:
		void firerocket(float quantum);
		void firebullet();
		float debristime;
		BITMAP * panel;
		BITMAP * panelbackground;
		void drawpanel();
		void updatepanel();
		bool panelneedsupdate;
		float burnrate; // rate at which we use fuel (per second)
		float firetime; // Timer until we can next fire.
		float ammorate; // rate at which we use ammo (per shot)
		void checkforitems(float quantum);
		GameObject *connecteditem;
		bool criticalhit;
		float criticaltimer;
		void blowup();
		float endlevel; // This needs to reach a certain level to end the level.
};

class Particle: public GameObject {
	public:
		float life; // Seconds
		int colour;
		Particle(); // Default constructor.
		virtual void draw();
		virtual void tick(float quantum);
		virtual ~Particle();
};

class Explosion: public Particle {
	public:
		Explosion(); // Default constructor.
		virtual void draw();
		virtual ~Explosion();
		BITMAP * sprite;
};

enum StaticType {
	PlayerBaseExit,
	TankHydrogen,
	TankOxygen,
	TankHCl,
	TankHF,
	PlayerBaseStart
};

// Things like bases, fuel tanks.
class StaticObject: public GameObject {
	public:
		StaticObject(StaticType s);
		StaticType type;
		BITMAP *sprite;
		virtual void draw();
		virtual ~StaticObject();
		virtual void save(ostream &out);
		int width;
		int height;
		bool drawn;
		void drawtocollisionmap();
		float capacity;
};

