#include <vector>
#include <allegro.h>
#include <math.h>
#include <iostream>

using namespace std;

#include "common.h"
#include "classes.h"
#include "data.h"
#include "manager.h"
#include "enemies.h"
#include "shot.h"
#include "playerinfo.h"

static const float CollisionDamage = 75;
static const float EnemyShotDamage = 50;
static const float MaxRange = 500;
static const float EnemyShotSpeed = 120;

/*
 * Enemies
 */

 Enemy::Enemy(EnemyType newtype)
 {
 	dead = false;
 	type = newtype;
	int spriteid = -1;
	mass = 5000;
	colour = makecol(0,0,255);
	switch (type) {
		case BlueStaticJelly:
			spriteid = JELLY_BLUE1_TGA;
			break;
		case OrangeJelly:
			spriteid = JELLY_ORANGE_TGA;
			colour = makecol(255,192,0);
			break;
		case YellowJelly:
			spriteid = JELLY_YELLOW_TGA;
			colour = makecol(255,255,0);
			break;
		case GreenJelly:
			spriteid = JELLY_GREEN_TGA;
			colour = makecol(0,255,0);
			break;
	}
	sprite = (BITMAP *) datafile[spriteid].dat;
	// Set up the radius correctly.
	radius = (sprite->w / 2);
	velocity.x = velocity.y = 0;
	targetvelocity.x = targetvelocity.y = 0;
	hp = (float) ((int) type) * 1.5;
	thinktime = 0;
	started = false;
	maxspeed = 50; // Pixels per sec?
	maxaccel = 80; // Pixels per sec per sec, gravity = 40
	maxammo = (int) type;
	ammo = maxammo;
	maxfiretime = 0.5;
	maxreloadtime = 5;
	firetime = 0;
}

void Enemy::draw()
{
	int xoffset = - (sprite->w / 2);
	int yoffset = - (sprite->h / 2);
	int px = ((int) position.x) + xoffset;
	int py = ((int) position.y) + yoffset;
	draw_sprite(vscreen, sprite, px,py);
	addrect(px,py,sprite->w, sprite->h);
}

void Enemy::tick(float quantum)
{
	CollidesWithScenery::tick(quantum);
	thinktime -= quantum;

	if (targetvelocity.x < velocity.x)
		velocity.x -= maxaccel * quantum;
	else
		velocity.x += maxaccel * quantum;
	if (targetvelocity.y < velocity.y)
		velocity.y -= maxaccel * quantum;
	else
		velocity.y += maxaccel * quantum;
	if (thinktime <0) think();
	firetime -= quantum;
	if (firetime <0) fire();
}

void Enemy::fire()
{
	// Player is already dead?
	if (! playership)
		return;
	if (ammo <= 0)
	{
		firetime = maxreloadtime;
		ammo = maxammo;
		return;
	}
	float dx, dy;
	dx = playership->position.x - position.x;
	dy = playership->position.y - position.y;
	float distsq = (dx * dx) + (dy * dy);
	if (distsq > MaxRange * MaxRange)
		return;
	// ok, now we've established that we want to fire.
	float dist = sqrt(distsq);
	dx /= dist;
	dy /= dist;
	Shot * s = new Shot(this);
	s->position.x = position.x;
	s->position.y = position.y;
	s->velocity.x = dx * EnemyShotSpeed;
	s->velocity.y = dy * EnemyShotSpeed;
	if (type == GreenJelly)
	{
		s->velocity.x *= 2;
		s->velocity.y *= 2;
	}
	s->colour = colour;
	s->damage = EnemyShotDamage;
	addobject(s);
	firetime = maxfiretime;
	ammo--;
}

void Enemy::think()
{
	if (! started) {
		targetpos = position;
		started = true;
	}
	float dx, dy;
	dx = targetpos.x - position.x;
	dy = targetpos.y - position.y;
	float distsq = (dx*dx) + (dy*dy);
	if (distsq > 10) {
		float dist = sqrt(distsq);
		dx = dx / dist;
		dy = dy / dist;
		targetvelocity.x = dx * maxspeed;
		targetvelocity.y = dy * maxspeed;
	}
	thinktime = randombetween(10,20) / 10;

}

void Enemy::save(ostream &out)
{
	int typenum = (int) type;
	out << "E " << typenum << " " << (int) position.x << " " << (int) position.y << endl;
}

void Enemy::takehit(float strength)
{
	float damage = strength/40;
	if (damage > 1.0) {
		hp = hp - (damage / 4);
		if (hp < 0) {
			// Dead!
			// Normal pitch, full vol.
			play_sound(SPLAT2_WAV, 1000, 255);
			playerinfo.score += ((int) type) * 20;
			playerinfo.money += ((int) type) * 20;
			for (int i=0; i<10; i++) {
				Explosion * e = new Explosion();
				e->position.x = position.x + randombetween((int) -radius, (int) radius);
				e->position.y = position.y + randombetween((int) -radius, (int) radius);
				e->sprite = (BITMAP *) datafile[SPLODGE_TGA].dat;
				addobject(e);
				for (int j=0; j<4; j++) {
					Particle * p = new Particle();
					p->position.x = e->position.x;
					p->position.y = e->position.y;
					p->velocity.x = randombetween(-10,10);
					p->velocity.y = randombetween(-10,10);
					p->colour = colour;
					addobject(p);
				}
			}

			dead = true;
		}
	}
}

void Enemy::collidewithother(CollidesWithScenery * other, float strength)
{
	// Check if it's an enemy, only hurt it if it is not.
	Enemy * e = dynamic_cast <Enemy *> (other);
	if (e == 0) {
		other->takehit(CollisionDamage); // Made up constant
	}
}

Enemy::~Enemy()
{
	// nothing
}
